# as_json specialised for OpenAI

    Code
      as_json(stub, type_object(.additional_properties = TRUE))
    Condition
      Error in `method(as_json, list(ellmer::ProviderOpenAICompatible, ellmer::TypeObject))`:
      ! `.additional_properties` not supported for OpenAI.

