# virtual_key is deprecated

    Code
      chat <- chat_portkey(model = "def", virtual_key = "abc")
    Condition
      Warning:
      The `virtual_key` argument of `chat_portkey()` is deprecated as of ellmer 0.4.0.
      i Please use the `model` argument instead.

