% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{has_credentials}
\alias{has_credentials}
\title{Are credentials avaiable?}
\usage{
has_credentials(provider)
}
\arguments{
\item{provider}{Provider name.}
}
\description{
Used for examples/testing.
}
\keyword{internal}
