% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/provider-google-tools.R
\name{google_tool_web_search}
\alias{google_tool_web_search}
\title{Google web search (grounding) tool}
\usage{
google_tool_web_search()
}
\description{
Enables Gemini models to search the web for up-to-date information and ground
responses with citations to sources. The model automatically decides when
(and how) to search the web based on your prompt. Search results are
incorporated into the response with grounding metadata including source
URLs and titles.

Learn more in \url{https://ai.google.dev/gemini-api/docs/google-search}.
}
\examples{
\dontrun{
chat <- chat_google_gemini()
chat$register_tool(google_tool_web_search())
chat$chat("What was in the news today?")
chat$chat("What's the biggest news in the economy?")
}
}
\seealso{
Other built-in tools:
\code{\link[=claude_tool_web_fetch]{claude_tool_web_fetch()}},
\code{\link[=claude_tool_web_search]{claude_tool_web_search()}},
\code{\link[=google_tool_web_fetch]{google_tool_web_fetch()}},
\code{\link[=openai_tool_web_search]{openai_tool_web_search()}}
}
\concept{built-in tools}
