% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/content-pdf.R
\name{content_pdf_file}
\alias{content_pdf_file}
\alias{content_pdf_url}
\title{Encode PDFs content for chat input}
\usage{
content_pdf_file(path)

content_pdf_url(url)
}
\arguments{
\item{path, url}{Path or URL to a PDF file.}
}
\value{
A \code{ContentPDF} object
}
\description{
These functions are used to prepare PDFs as input to the chatbot. The
\code{content_pdf_url()} function is used to provide a URL to an PDF file,
while \code{content_pdf_file()} is used to for local PDF files.

Not all providers support PDF input, so check the documentation for the
provider you are using.
}
