% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ellmer-package.R
\docType{package}
\name{ellmer-package}
\alias{ellmer}
\alias{ellmer-package}
\title{ellmer: Chat with Large Language Models}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Chat with large language models from a range of providers including 'Claude' \url{https://claude.ai}, 'OpenAI' \url{https://chatgpt.com}, and more. Supports streaming, asynchronous calls, tool calling, and structured data extraction.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://ellmer.tidyverse.org}
  \item \url{https://github.com/tidyverse/ellmer}
  \item Report bugs at \url{https://github.com/tidyverse/ellmer/issues}
}

}
\author{
\strong{Maintainer}: Hadley Wickham \email{hadley@posit.co} (\href{https://orcid.org/0000-0003-4757-117X}{ORCID})

Authors:
\itemize{
  \item Hadley Wickham \email{hadley@posit.co} (\href{https://orcid.org/0000-0003-4757-117X}{ORCID})
  \item Joe Cheng
  \item Aaron Jacobs
  \item Garrick Aden-Buie \email{garrick@posit.co} (\href{https://orcid.org/0000-0002-7111-0077}{ORCID})
  \item Barret Schloerke \email{barret@posit.co} (\href{https://orcid.org/0000-0001-9986-114X}{ORCID})
}

Other contributors:
\itemize{
  \item Posit Software, PBC (\href{https://ror.org/03wc8by49}{ROR}) [copyright holder, funder]
}

}
\keyword{internal}
