% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/provider-google-tools.R
\name{google_tool_web_fetch}
\alias{google_tool_web_fetch}
\title{Google URL fetch tool}
\usage{
google_tool_web_fetch()
}
\description{
When this tool is enabled, you can include URLs directly in your prompts and
Gemini will fetch and analyze the content.

Learn more in \url{https://ai.google.dev/gemini-api/docs/url-context}.
}
\examples{
\dontrun{
chat <- chat_google_gemini()
chat$register_tool(google_tool_web_fetch())
chat$chat("What are the latest package releases on https://tidyverse.org/blog?")
}
}
\seealso{
Other built-in tools:
\code{\link[=claude_tool_web_fetch]{claude_tool_web_fetch()}},
\code{\link[=claude_tool_web_search]{claude_tool_web_search()}},
\code{\link[=google_tool_web_search]{google_tool_web_search()}},
\code{\link[=openai_tool_web_search]{openai_tool_web_search()}}
}
\concept{built-in tools}
